mod Maux

incl Mtex

incl Mtexar

rem 
  \para \bf  Section. Formalization of Arrays.  \end 

rem 
  \para \bf  Subsection. Basic Operations.  \end 

rem 
  \para \bf  Array length.  \end 

fun L 
  L(0) = 0
  L(v,w) = L(w)+1

rem 
  \para \bf  Arrays.  \end 

pred/2 A 'Tex_p2_a_array'
  A(a,n) <-> L(a) = n

rem 
  \para \bf  Array subscription.  \end 

fun/2 Sub 'Tex_f2_a_sub'
  Sub((v,w),0) = v
  Sub((v,w),i+1) = Sub(w,i)

rem 
  \para \bf  Array modification.  \end \header* fun/3 Msub 'Tex_f3_a_msub' \end 
  Specification: 
  \eq* 
    A(a,n) & i < n -> A(Msub(a,i,x),n)
  \end
  \eq* 
    A(a,n) & i < n -> Sub(Msub(a,i,x),i) = x
  \end
  \eq* 
    A(a,n) & i < n & j < n & j != i -> Sub(Msub(a,i,x),j) = Sub(a,j)
  \end

fun/3 Msub 'Tex_f3_a_msub'
  Msub((v,w),0,x) = x,w
  Msub((v,w),i+1,x) = v,Msub(w,i,x)

rem 
  \para \bf  New array.  \end Specification: 
  \eq* 
    A(New(n),n)
  \end

fun New 
  New(0) = 0
  New(n+1) = 0,New(n)

rem 
  \para \bf  Subsection. Additional Operations.  \end 

rem 
  \para \bf  Swapping two elements.  \end 
  \header* fun/3 Swap 'Tex_f3_a_swap' \end Specification: 
  \eq* 
    A(a,n) & i < n & j < n -> A(Swap(a,i,j),n)
  \end
  \eq* 
    A(a,n) & i < n & j < n -> Sub(Swap(a,i,j),i) = Sub(a,j)
  \end
  \eq* 
    A(a,n) & i < n & j < n -> Sub(Swap(a,i,j),j) = Sub(a,i)
  \end
  \eq* 
    A(a,n) & i < n & j < n & k < n & k != i & k != j -> 
    Sub(Swap(a,i,j),k) = Sub(a,k)
  \end

fun/3 Swap 'Tex_f3_a_swap'
  Swap(a,i,j) = Msub(Msub(a,i,Sub(a,j)),j,x) <- Sub(a,i) = x

rem 
  \para \bf  Array concatenation.  \end \header* fun/2 Conc 'Tex_f2_a_conc' \end 
  Specification: 
  \eq* 
    A(a,m) & A(b,n) -> A(Conc(a,b),m+n)
  \end
  \eq* 
    A(a,m) & A(b,n) & i < m -> Sub(Conc(a,b),i) = Sub(a,i)
  \end
  \eq* 
    A(a,m) & A(b,n) & i < n -> Sub(Conc(a,b),i+m) = Sub(b,i)
  \end
  \para Definition: 
  \def 
    fun/2 Conc 'Tex_f2_a_conc'
    Conc(0,y) = y
    Conc((v,w),y) = v,Conc(w,y)
  \end
  \para \it  [CL] Remark.  \end Note that the concatenation function is builtin 
  in CL. 

rem 
  \para \bf  Subarrays.  \end \header* fun/3 Sba 'Tex_f3_a_sba' \end 
  Specification: 
  \eq* 
    A(a,n) & k <= m & m <= n -> A(Sba(a,k,m),m-k)
  \end
  \eq* 
    A(a,n) & k <= m & m <= n & k+i < m -> Sub(Sba(a,k,m),i) = Sub(a,k+i)
  \end

fun/2 Take 
  Take(0,a) = 0
  Take(n+1,v,w) = v,Take(n,w)

fun/2 Drop 
  Drop(0,a) = a
  Drop(n+1,v,w) = Drop(n,w)

fun/3 Sba 'Tex_f3_a_sba'
  Sba(a,k,m) = Drop(k,Take(m,a))

rem 
  \para \bf  Array membership.  \end Definition: 
  \def 
    pred/2 In1 'Tex_p2_a_in'
    In1(x,a) <-> \e i(i < L(a) & Sub(a,i) = x)
  \end
  \para \it  [CL] Remark.  \end Note that the membership predicate is builtin in 
  CL. 

rem 
  \para \bf  Ordered arrays.  \end 

pred Ord 
  Ord(a) <-> \a i\a j(i < j & j < L(a) -> Sub(a,i) <= Sub(a,j))

rem 
  \para \bf  Permutation of arrays.  \end 

fun/2 Eqf 'Tex_f2_char_eqf'
  Eqf(x,y) = 1 <- x = y
  Eqf(x,y) = 0 <- x != y

fun/2 Count 'Tex_f2_ln_count'
  Count(x,0) = 0
  Count(x,v,w) = Count(x,w)+Eqf(x,v)

pred/2 Perm 'Tex2_sim'
  Perm(a,b) <-> \a x Count(x,a) = Count(x,b)

